<?php
/**
 * @file
 * Defines the ControllerMenuItemsCollector and utility classes.
 */

require_once drupal_get_path('module', 'controller') . '/services/NamingService.php';

/**
 * This Collector collects menu items that were defined
 * in Controllers.
 */
class ControllerMenuItemsCollector {
  /**
   * Returns lists of menu items defined inside classes that
   * reside in the module directory.
   */
  function getMenuItemsForModuleDir($module_name, $dir_path) {
    $mask = '{\.php$}si';
    $module_dir_path = drupal_get_path('module', $module_name);
    $controllers_dir_path = trim($dir_path, '\\/');
    $full_dir_path = $module_dir_path . '/' . $controllers_dir_path;
    $files = file_scan_directory($full_dir_path, $mask);
    $menu_items = array();
    foreach ($files as $file) {
      require_once $file->uri;
      $controller_name = $file->name;
      $class_name = ControllerNamingService::createFullClassName(
        $module_name,
        $controller_name
      );
      if (!class_exists($class_name)) {
        $vars = array(
          '%fileName' => $file->filename,
          '%className' => $class_name,
        );
        $message = t("The controller class '%className' was not found in the file '%fileName'.", $vars);
        watchdog(CONTROLLER_WATCHDOG_ID, $message, array(), WATCHDOG_ERROR);
        drupal_set_message($message, 'error');
        continue;
      }
      $class_meta_data = $this->createClassMetaData($class_name);
      $menu_items = array_merge(
        $menu_items,
        $this->collectMenuItemsFromClass(
          $class_meta_data,
          $module_name,
          $controller_name,
          $module_dir_path,
          $controllers_dir_path
        )
      );
    }
    return $menu_items;
  }

  /**
   * Returns lists of menu items defined inside classes that reside
   * in the module's controllers/ directory.
   */
  function getMenuItemsForModule($module_name) {
    return $this->getMenuItemsForModuleDir($module_name, CONTROLLER_DIR_NAME);
  }

  /**
   * Creates the meta-data object for class.
   */
  protected function createClassMetaData($class_name) {
    return new ControllerClassMetaData($class_name);
  }

  /**
   * Collects menu items that defined inside the specifed class.
   */
  protected function collectMenuItemsFromClass($class_meta_data, $module_name, $controller_name, $module_dir_path, $controllers_dir_path) {
    $menu_items = array();
    $class_name = $class_meta_data->getName();
    foreach ($class_meta_data->getActionList() as $method_meta_data) {
      $method_name = $method_meta_data->getName();
      $items = $this->collectMenuItemFromMethod($method_meta_data);
      if (!isset($items['path'])) {
        // It is not action method.
        continue;
      }
      $path = $items['path'];
      unset($items['path']);
      // Add the new special item 'action' that will be handled
      // by the controller_normalize_menu_items() function later.
      $items['action'] = $controller_name . CONTROLLER_MENU_SEPARATOR . $method_name;
      $items['module'] = $module_name;
      $items['file path'] = $module_dir_path;
      $items['file'] = $controllers_dir_path . '/' . $controller_name . '.php';
      $menu_items[$path] = $items;
    }
    return $menu_items;
  }

  /**
   * Allows to collect menu items that defined as php comments.
   */
  protected function collectMenuItemFromMethod($method_meta_data) {
    $docs = $method_meta_data->getDocComment();
    if (FALSE === $pos = strpos($docs, '@path')) {
      return array();
    }
    $chunks = preg_split('{^\\s*\\*[\\\\\\s]*@}mi', $docs);
    $items = array();
    $keys = array(
      // New special items introduced by the 'controller' module.
      'path',
      'default',
      // Standard Drupal items.
      'title callback',
      'title arguments',
      'title',
      'description',
      'page callback',
      'page arguments',
      'access callback',
      'access arguments',
      'file path',
      'file',
      'load arguments',
      'weight',
      'menu_name',
      'tab_parent',
      'tab_root',
      'position',
      'type',
    );
    $regexp = '{^(' . implode('|', $keys) . ')\\s*=>\\s*(.*)}si';
    foreach ($chunks as $chunk) {
      $chunk = trim($chunk);
      if (preg_match($regexp, $chunk, $m)) {
        $value = array_pop($m);
        $key = array_pop($m);

        $value = preg_replace(array('{^[\s*]+}mi', '{[\s,;*/]+$}mi'), array('', ''), $value);
        $value = eval('?><?php return ' . $value . ';');
        $items[$key] = $value;
      }
    }
    return $items;
  }
}

/**
 * Used to provide meta-data for the specified controller class.
 */
class ControllerClassMetaData extends ReflectionClass {
  protected $moduleName;

  /**
   * Used to get list of action methods.
   */
  public function getActionList() {
    $action_list = array();
    foreach ($this->getMethods(ReflectionMethod::IS_PUBLIC) as $method) {
      $action_list[] = new ControllerMethodMetaData($method);
    }
    return $action_list;
  }
}

/**
 * Used to provide meta-data for the specified controller's method.
 */
class ControllerMethodMetaData {
  /**
   * Constructor
   */
  function __construct(ReflectionMethod $method) {
    $this->reflectionMethod = $method;
  }

  /**
   * Magic method to delegate resposibility to the inner object.
   */
  function __call($method, $args) {
    return call_user_func_array(array($this->reflectionMethod, $method), $args);
  }
}
