<?php
/**
 * @file
 * Defines the ControllerMenuItemsNormalizer and utility classes.
 */

require_once drupal_get_path('module', 'controller') . '/services/NamingService.php';

/**
 * The ControllerMenuItemsNormalizer do normalization of menu items
 * to follow Drupal's conventions.
 */
class ControllerMenuItemsNormalizer {
  /**
   * Makes normalization of menu items.
   */
  function normalizeMenuItems(array &$items) {
    foreach ($items as $path => &$item) {
      if (isset($item[CONTROLLER_MENU_ACTION_KEY])) {
        $new_items = $this->normalizeMenuItem($item, $path);
        $items = array_merge($items, $new_items);
      }
    }
    unset($item);
  }

  /**
   * Normalizes single menu item.
   */
  function normalizeMenuItem(array $item, $path) {
    $menu_item = new ControllerExtendedMenuItem($item, $path);
    return $menu_item->toArray();
  }
}

/**
 * Represents base class for all menu items.
 */
abstract class ControllerBaseMenuItem {
  protected $defaultItems = array(
    'title' => '',
    'weight' => 0,
  );

  protected $requiredKeys = array(
    'module',
  );

  protected $menuItemDef;

  protected $path;

  /**
   * Constructor.
   */
  function __construct(array $menu_item_def, $path) {
    $this->path = $path;
    $this->checkRequiredKeys($menu_item_def);
    $menu_item_def = array_merge($this->getDefaultItems(), $menu_item_def);
    $this->menuItemDef = $this->transform($menu_item_def);
  }

  /**
   * Converts object to array.
   */
  function toArray() {
    return array($this->path => $this->menuItemDef);
  }

  /**
   * Returns menu type.
   */
  abstract protected function getType();

  /**
   * Returns default values for menu item definition.
   */
  protected function getDefaultItems() {
    return $this->defaultItems + array('type' => $this->getType());
  }

  /**
   * Transforms input menu definition.
   */
  protected function transform(array $menu_item_def) {
    return $menu_item_def;
  }

  /**
   * Returns required keys that should always exist
   * in the menu definition.
   */
  protected function getRequiredKeys() {
    return $this->requiredKeys;
  }

  /**
   * Checks that all required keys exist in the menu item
   * definition.
   */
  protected function checkRequiredKeys(array $menu_item_def) {
    foreach ($this->getRequiredKeys() as $key_name) {
      if (!isset($menu_item_def[$key_name])) {
        throw new RuntimeException('Missing key was found: ' . check_plain($key_name));
      }
    }
  }
}

/**
 * Represents the MENU_NORMAL_ITEM.
 */
class ControllerNormalMenuItem extends ControllerBaseMenuItem {
  /**
   * Returns default values for menu item definition.
   */
  protected function getDefaultItems() {
    return parent::getDefaultItems() + array('page arguments' => array());
  }

  /**
   * Returns menu type.
   */
  protected function getType() {
    return MENU_NORMAL_ITEM;
  }
}

/**
 * Represents the MENU_DEFAULT_LOCAL_TASK.
 */
class ControllerDefaultLocalTaskMenuItem extends ControllerBaseMenuItem {
  /**
   * Returns menu type.
   */
  protected function getType() {
    return MENU_DEFAULT_LOCAL_TASK;
  }
}

/**
 * Represents the customized menu item that can be used
 * for definition of the action methods.
 */
class ControllerExtendedMenuItem extends ControllerNormalMenuItem {
  protected $defaultLocalTaskMenuItem;

  /**
   * Converts object to array.
   */
  public function toArray() {
    $menu_item_def = parent::toArray();
    if (NULL !== $this->defaultLocalTaskMenuItem) {
      $menu_item_def = array_merge(
        $menu_item_def,
        $this->defaultLocalTaskMenuItem->toArray()
      );
    }
    return $menu_item_def;
  }

  /**
   * Returns required keys that should always exist
   * in the menu definition.
   */
  protected function getRequiredKeys() {
    return array_merge(parent::getRequiredKeys(), array('action'));
  }

  /**
   * Returns default values for menu item definition.
   */
  protected function getDefaultItems() {
    return parent::getDefaultItems() + array(
      'default' => FALSE,
    );
  }

  /**
   * Transforms input menu definition.
   */
  protected function transform(array $menu_item_def) {
    $parts = explode(CONTROLLER_MENU_SEPARATOR, $menu_item_def[CONTROLLER_MENU_ACTION_KEY]);
    // If no action method was defined, use 'indexAction'.
    if (empty($parts[1])) {
      $parts[1] = CONTROLLER_DEFAULT_ACTION;
    }
    $controller_name = $parts[0];
    $action_name = $parts[1];
    unset($menu_item_def[CONTROLLER_MENU_ACTION_KEY]);

    $module_name = $menu_item_def['module'];

    $controller_class_name = ControllerNamingService::createFullClassName(
      $module_name,
      $controller_name
    );

    if (!isset($menu_item_def['file'])) {
      $menu_item_def['file'] = CONTROLLER_DIR_NAME . '/' . $controller_name . '.php';
    }

    $menu_item_def['page callback'] = CONTROLLER_MENU_HANDLER;
    $menu_item_def['page arguments'] = array_merge(
      array(
        $controller_class_name
        . CONTROLLER_MENU_SEPARATOR
        . $action_name
        . CONTROLLER_MENU_SEPARATOR
        . $menu_item_def['module'],
      ),
      $menu_item_def['page arguments']
    );

    if ($menu_item_def['default'] !== FALSE) {
      $this->defaultLocalTaskMenuItem = $this->createDefaultLocalTaskMenuItem($menu_item_def);
    }
    unset($menu_item_def['default']);

    return $menu_item_def;
  }

  /**
   * Creates an instance of ControllerDefaultLocalTaskMenuItem
   * for this menu item.
   */
  protected function createDefaultLocalTaskMenuItem(array $menu_item_def) {
    if (!is_array($menu_item_def['default'])) {
      throw new \RuntimeException("Invalid format the menu item: " . check_plain(print_r($menu_item_def, 1)));
    }
    $local_task_menu_item_def = array(
      'file' => $menu_item_def['file'],
      'module' => $menu_item_def['module'],
    );
    $key = key($menu_item_def['default']);
    $path = $this->path . '/' . trim($key, '/');
    $local_task_menu_item_def['title'] = $menu_item_def['default'][$key];
    return new ControllerDefaultLocalTaskMenuItem(
      $local_task_menu_item_def,
      $path
    );
  }
}
